# ALL FUNCTIONS IN THIS FILE ARE FROM HANDLING DATA. NO NEED TO REVIEW

import re
from typing import Tuple, List, Union

from sohstationviewer.conf.dbSettings import dbConf


def check_chan(chan_id: str, req_soh_chans: List[str], req_wf_chans: List[str],
               include_mp123zne: bool, include_mp456uvw: bool) \
        -> Union[str, bool]:
    """
    Check if chanID is a requested channel.
    :param chan_id: str - channel ID
    :param req_soh_chans: list of str - requested SOH channels
    :param req_wf_chans: list of str - requested waveform channels
    :param include_mp123zne: if mass position channels 1,2,3 are requested
    :param include_mp456uvw: if mass position channels 4,5,6 are requested

    :return: str/bool -
        'WF' if chanID is a requested waveform channel,
        'SOH' if chanID is a requested SOH channel,
        'MP' if chanID is a requested mass position channel
        False otherwise.
    """
    if chan_id.startswith('VM'):
        if (not include_mp123zne and
                chan_id[-1] in ['1', '2', '3', 'Z', 'N', 'E']):
            return False
        if (not include_mp456uvw
                and chan_id[-1] in ['4', '5', '6', 'U', 'V', 'W']):
            return False
        return 'MP'

    ret = check_wf_chan(chan_id, req_wf_chans)
    if ret[0] == 'WF':
        if ret[1]:
            return "WF"
        else:
            return False
    if check_soh_chan(chan_id, req_soh_chans):
        return "SOH"
    return False


def check_soh_chan(chan_id: str, req_soh_chans: List[str]) -> bool:
    """
    Check if chan_id is a requested SOH channel.
    Mass position is always included.
    This function is used for mseed only so mass position is 'VM'.
    If there is no reqSOHChans, it means all SOH channels are requested
    :param chan_id: str - channel ID
    :param req_soh_chans: list of str - requested SOH channels
    :return: bool - True if chan_id is a requested SOH channel. False otherwise
    """
    if req_soh_chans == []:
        return True
    if chan_id in req_soh_chans:
        return True
    if 'EX?' in req_soh_chans and chan_id.startswith('EX'):
        if chan_id[2] in ['1', '2', '3']:
            return True
    # TODO: remove mass position channels from SOH
    if chan_id.startswith('VM'):
        if chan_id[2] in ['0', '1', '2', '3', '4', '5', '6']:
            return True
    return False


def check_wf_chan(chan_id: str, req_wf_chans: List[str]) -> Tuple[str, bool]:
    """
    Check if chanID is a waveform channel and is requested by user
    :param chan_id: str - channel ID
    :param req_wf_chans: list of str - requested waveform channels
    :return wf: str - '' if chan_id is not a waveform channel.
                      'WF' if chan_id is a waveform channel.
    :return has_chan: bool - True if chan_id is a requested waveform channel.
    """
    if not dbConf['seisRE'].match(chan_id):
        return '', False

    for req in req_wf_chans:
        if len(req) == 1:
            req = req.replace('*', '...')
        elif len(req) == 2:
            req = req.replace('*', '..')
        elif len(req) == 3:
            req = req.replace('*', '.')

        if re.compile(f'^{req}$').match(chan_id):
            return 'WF', True

    return 'WF', False
